local *

{
  :ChatFrame_AddMessageEventFilter
  :GetCoinTextureString
  :InterfaceOptionsFrame_OpenToCategory
  :PVEFrame_ShowFrame
  :ReloadUI
  :SlashCmdList
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :ToggleOption
} = idTweaksOptions

class Chat extends idTweaksModule
  new: (addon) =>
    -- print "Chat#new"

    super addon, "Chat", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0"

  set_options: =>
    -- print "Chat#set_options"

    super!

    @options\add_option Option @, "slash_commands", "Slash Commands", "Customize the Slash Commands", {},
      addon: ToggleOption @, "addon", "/idtweaks (/idt)", "Open the idTweaks Options with /idtweaks and /idt",
      reload: ToggleOption @, "reload", "/reload (/rl)", "Reload the UI with /reload and /rl"
      lfg: ToggleOption @, "lfg", "/lfg", "Open the Group Finder with /lfg"

    @options\add_option Option @, "message_style", "Message Style", "Customize the chat message styles", {},
      enabled: ToggleOption @, "enabled", "Enabled", "Change the style of your chat messages to a minimal format"

  check: =>
    -- print "Chat#check"

    @slash_commands_check!
    @message_style_check!

  enable: =>
    -- print "Chat#enable"

    @check!

  disable: =>
    -- print "Chat#disable"

    @slash_commands_disable!
    @message_style_disable!

  slash_commands_check: =>
    -- print "Chat#slash_commands_check"

    if @db.slash_commands.addon
      -- print "Chat#slash_commands_check:enabling-addon"
      SlashCmdList["IDTWEAKS_ADDON"] = (msg) ->
        @ace_module\Print "Opening options"
        InterfaceOptionsFrame_OpenToCategory "idTweaks"
        InterfaceOptionsFrame_OpenToCategory "idTweaks"
        InterfaceOptionsFrame_OpenToCategory "Automation"
      _G.SLASH_IDTWEAKS_ADDON1 = "/idt"
      _G.SLASH_IDTWEAKS_ADDON2 = "/idtweaks"
    else
      -- print "Chat#slash_commands_check:disabling-addon"
      _G.SlashCmdList["IDTWEAKS_ADDON"] = nil
      _G.SLASH_IDTWEAKS_ADDON1 = nil
      _G.SLASH_IDTWEAKS_ADDON2 = nil

    if @db.slash_commands.reload
      -- print "Chat#slash_commands_check:enabling-reload"
      _G.SlashCmdList["IDTWEAKS_RELOAD"] = ReloadUI
      _G.SLASH_IDTWEAKS_RELOAD1 = "/rl"
      _G.SLASH_IDTWEAKS_RELOAD2 = "/reload"
    else
      -- print "Chat#slash_commands_check:disabling-reload"
      _G.SlashCmdList["IDTWEAKS_RELOAD"] = nil
      _G.SLASH_IDTWEAKS_RELOAD1 = nil
      _G.SLASH_IDTWEAKS_RELOAD2 = nil

    if @db.slash_commands.lfg
      -- print "Chat#slash_commands_check:enabling-lfg"
      _G.SlashCmdList["IDTWEAKS_LFG"] = ->
        PVEFrame_ShowFrame "GroupFinderFrame"
      _G.SLASH_IDTWEAKS_LFG1 = "/lfg"
    else
      -- print "Chat#slash_commands_check:disabling-lfg"
      _G.SlashCmdList["IDTWEAKS_LFG"] = nil
      _G.SLASH_IDTWEAKS_LFG1 = nil

  message_style:
    formatters:
      money: (message) ->
        gold = message\match "(%d+) Gold"
        silver = message\match "(%d+) Silver"
        copper = message\match "(%d+) Copper"

        gold = string.format "%02d", gold if gold
        silver = string.format "%02d", silver if silver
        copper = string.format "%02d", copper if copper

        total = ""
        total = total .. gold if gold
        total = total .. silver if silver
        total = total .. copper if copper

        "+ #{GetCoinTextureString total}"

    filters:
      CHAT_MSG_AFK: {}
      CHAT_MSG_COMBAT_FACTION_CHANGE:
        "Reputation with (.+) increased by (.+).": "+ %2 rep (%1)"
      CHAT_MSG_COMBAT_HONOR_GAIN: {}
      CHAT_MSG_COMBAT_MISC_INFO: {}
      CHAT_MSG_CURRENCY:
        "You receive currency: (.+)": (message) -> Chat.message_style.formatters.money message
      CHAT_MSG_DND: {}
      CHAT_MSG_GUILD_ACHIEVEMENT:
        "%%s has earned the achievement (.+)!": "\124cff00ff00g %1 %%s\124h"
      CHAT_MSG_IGNORED: {}
      CHAT_MSG_LOOT:
        "You receive item: (.+)": "+ %1"
        "You receive loot: (.+)": "+ %1"
        "Received item: (.+)": "+ %1"
      CHAT_MSG_MONEY:
        "Received (.+) Gold, (.+) Silver, (.+) Copper": (message) -> Chat.message_style.formatters.money message
        "Received (.+) Silver, (.+) Copper": (message) -> Chat.message_style.formatters.money message
        "Received (.+) Silver": (message) -> Chat.message_style.formatters.money message
        "Received (.+) Copper": (message) -> Chat.message_style.formatters.money message
        "You loot (.+) Gold, (.+) Silver, (.+) Copper": (message) -> Chat.message_style.formatters.money message
        "You loot (.+) Silver, (.+) Copper": (message) -> Chat.message_style.formatters.money message
        "You loot (.+) Silver": (message) -> Chat.message_style.formatters.money message
        "You loot (.+) Copper": (message) -> Chat.message_style.formatters.money message
      CHAT_MSG_SKILL:
        "Your skill in (.+) has increased to (.+).": "+ %2 (%1)"
      CHAT_MSG_SYSTEM:
        "Received (.+) Silver": "+ %1 s"
        "Received (.+) of item: (.+).": "+ %2x%1"
        "(.+) completed.": "- quest (%1)"
        "Quest accepted: (.+)": "+ quest (%1)"
        "Received item: (.+)%.": "+ %1"
        "Experience gained: (.+).": "+ %1 xp"
        "(.+) has come online.": "+ %1"
        "(.+) has gone offline.": "- %1"
        "You are now Busy: in combat": "+ Combat"
        "You are no longer marked Busy.": "- Combat"
        "Discovered (.+): (.+) experience gained": "+ %2 xp (%1)"
        "Quest Accepted (.+)": "+ quest (%1)"
        "You are now Away: AFK": "+ AFK"
        "You are no longer Away.": "- AFK"
        "You are no longer rested.": "- Rested"
        "You don\"t meet the requirements for that quest.": "! quest (Requirements not met)"
      CHAT_MSG_TARGETICONS: {}
      CHAT_MSG_TRADESKILLS: {}

  message_style_check: =>
    -- print "Chat#message_style_check"

    if @db.message_style.enabled
      @message_style_enable!
    else
      @message_style_disable!

  message_style_enable: =>
    -- print "Chat#message_style_enable"

    for event, event_filters in pairs @message_style.filters
      ChatFrame_AddMessageEventFilter event, (frame, ev, message, ...) ->
        for k, v in pairs event_filters
          if message\match k
            if (type v) == "function"
              message = v message
            else
              message = message\gsub k, v
            return nil, message, ...
        return nil, message, ...

  -- TODO: Implement.
  message_style_disable: =>
    -- print "Chat#message_style_disable"

idTweaks\add_module Chat idTweaks
